/*
 * rmote-coap-server.c
 *
 * Copyright (c) 2012, ROSAND Technologies
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the Institute nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE INSTITUTE AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE INSTITUTE OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * This file is part of the Contiki operating system.
 *
 */

/*
 * \file
 *      CoAP Server for R-Mote p8 Development Board.
 *      This implementation includes :
 *              - LED actuator
 *              - Environment Sensor event handler.
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "contiki.h"
#include "contiki-net.h"

#include "erbium.h"
#include "er-coap-07.h"

#include "leds.h"
#include "defs.h"
#include "rmote-coap-server.h"
#include "dev/rmote-sensors.h"

#define DEBUG 0
#if DEBUG
#define PRINTF(...) printf(__VA_ARGS__)
#define PRINT6ADDR(addr) PRINTF("[%02x%02x:%02x%02x:%02x%02x:%02x%02x:%02x%02x:%02x%02x:%02x%02x:%02x%02x]", ((uint8_t *)addr)[0], ((uint8_t *)addr)[1], ((uint8_t *)addr)[2], ((uint8_t *)addr)[3], ((uint8_t *)addr)[4], ((uint8_t *)addr)[5], ((uint8_t *)addr)[6], ((uint8_t *)addr)[7], ((uint8_t *)addr)[8], ((uint8_t *)addr)[9], ((uint8_t *)addr)[10], ((uint8_t *)addr)[11], ((uint8_t *)addr)[12], ((uint8_t *)addr)[13], ((uint8_t *)addr)[14], ((uint8_t *)addr)[15])
#define PRINTLLADDR(lladdr) PRINTF("[%02x:%02x:%02x:%02x:%02x:%02x]",(lladdr)->addr[0], (lladdr)->addr[1], (lladdr)->addr[2], (lladdr)->addr[3],(lladdr)->addr[4], (lladdr)->addr[5])
#else
#define PRINTF(...)
#define PRINT6ADDR(addr)
#define PRINTLLADDR(addr)
#endif

static char msg[REST_MAX_CHUNK_SIZE];
static uint16_t temperature, humidity, light;

/*----------------------------------------------------------------------*/
/*
 * A simple LED actuator.
 * Depending on the colour query parameter and post variable mode, corresponding
 * led is activated, de-activated or toggled.
 */
RESOURCE(leds, METHOD_GET | METHOD_POST | METHOD_PUT , "actuators/leds", "title=\"LEDs: ?color=r|g, POST/PUT mode=on|off|toggle\";rt=\"Control\"");

void
leds_handler(void* request, void* response, uint8_t *buffer, uint16_t preferred_size, int32_t *offset)
{
  size_t len = 0;
  const char *color = NULL;
  const char *mode = NULL;
  uint8_t led = 0;
  int success = 1;

  if ((len=REST.get_query_variable(request, "color", &color))) {
    PRINTF("color %.*s\n", len, color);

    if(strncmp(color, "r", len)==0) {
      led = LEDS_RED;
    } else if(strncmp(color,"g", len)==0) {
      led = LEDS_GREEN;
    } else if(strncmp(color,"y", len)==0) {
      led = LEDS_YELLOW;
    } else {
      success = 0;
    }
  } else {
    success = 0;
  }

  if(success && (len=REST.get_post_variable(request, "mode", &mode))) {
    PRINTF("mode %s\n", mode);

    if(strncmp(mode, "on", len)==0) {
      leds_on(led);
    } else if(strncmp(mode, "off", len)==0) {
      leds_off(led);
    }else if(strncmp(mode, "toggle", len)==0) {
      leds_toggle(led);
    } else {
      success = 0;
    }
  } else {
    if((led & leds_get())) {
      snprintf(msg, REST_MAX_CHUNK_SIZE, "led is on");
    } else {
      snprintf(msg, REST_MAX_CHUNK_SIZE, "led is off");
    }
    REST.set_response_payload(response, msg, strlen(msg));
  }

  if(!success) {
    REST.set_response_status(response, REST.status.BAD_REQUEST);
  }
}
/*----------------------------------------------------------------------*/
/*
 * handler: parse the sensor data into fields that can
 * be interpreted by the resource handler.
 */
static void
handler(char *buf)
{
  char len;
  char *tmp = buf;

  len  = *tmp++;
  do {
    char type;
    char lpar;
    char *st;

    type = *tmp++;
    lpar = *tmp++;
    st  =  tmp;
    tmp += lpar;

    switch(type) {
    case ENV_TEMP_SENSOR:
      temperature = BUILD_UINT16(*(st+1), *st);
      break;
    case ENV_HUMID_SENSOR:
      humidity = BUILD_UINT16(*(st+1), *st);
     break;
    case ENV_LUX_SENSOR:
      light = BUILD_UINT16(*(st+1), *st);
      break;
    }
  } while((tmp -buf) < len);

}
/*----------------------------------------------------------------------*/
/*
 * A simple event notification resource.
 * When the value of one of the Environment sensors changes,
 * notify the registered observers.
 */
EVENT_RESOURCE(env_sensor, METHOD_GET, "sensors/Environment sensor", "title=\"Environment Sensor\";obs");

void
env_sensor_handler(void* request, void* response, uint8_t *buffer, uint16_t preferred_size, int32_t *offset)
{
  const uint16_t *accept = NULL;
  int num = REST.get_header_accept(request, &accept);

  /* CoAP server responds with the current resource representation. */
  if ((num == 0) || (num && accept[0] == REST.type.TEXT_PLAIN)) {
    REST.set_header_content_type(response, REST.type.TEXT_PLAIN);
    snprintf(msg, REST_MAX_CHUNK_SIZE, "Temperature: %2.1f °C\nHumidity: %d %%\nLight: %d uW/cm²",
        (double) (temperature / 100), humidity, light);
  }
  else if (num && (accept[0] == REST.type.APPLICATION_XML)) {
    snprintf((char *) msg, REST_MAX_CHUNK_SIZE, "<env_sensor Temperature:\"%u °C\" Humidity:\"%u %%\" Light:\"%u uW/cm²\"/>",
        temperature, humidity, light);
  }
  else if (num && (accept[0] == REST.type.APPLICATION_JSON)) {
    REST.set_header_content_type(response, REST.type.APPLICATION_JSON);
    snprintf((char *)msg, REST_MAX_CHUNK_SIZE, "{'env_sensor':{'Temperature':%u °C,'Humidity':%u %%, 'Light':%u uW/cm²}}",
        temperature, humidity, light);
  }
  else {
    REST.set_response_status(response, REST.status.UNSUPPORTED_MADIA_TYPE);
    sprintf((char *) msg, "Supporting content-types text/plain, application/xml, and application/json");
  }
  REST.set_response_payload(response, msg, strlen(msg));
}
/*----------------------------------------------------------------------*/
/* Additionally, a handler function named [resource name]_event_handler is implemented.
 * It will be called by the REST manager process when the event happens. */
void
env_sensor_event_handler(resource_t *r)
{
  static uint16_t event_counter = 0;
  char content[18];

  ++event_counter;

  /* Build notification. */
  coap_packet_t notification[1]; /* This way the packet can be treated as pointer as usual. */
  coap_init_message(notification, COAP_TYPE_CON, CONTENT_2_05, 0 );
  snprintf(content, sizeof(content), "ENV_SENSOR %u\n", event_counter);
  coap_set_payload(notification, content, strlen(content));

  /* Notify the registered observers with the given message type, observe option, and payload. */
  REST.notify_subscribers(r, event_counter, notification);
}

/*----------------------------------------------------------------------*/
PROCESS(coap_server, "R-Mote p8 CoAP Server");
AUTOSTART_PROCESSES(&coap_server);

PROCESS_THREAD(coap_server, ev, data)
{
  PROCESS_BEGIN();

  /* Initialize the REST engine. */
  rest_init_engine();

  /* Activate Resource leds */
  rest_activate_resource(&resource_leds);

  /* Activate env_sensor resource */
  rest_activate_event_resource(&resource_env_sensor);

  /* Define application-specific events here. */
  while(1) {
    PROCESS_WAIT_EVENT();

    if(ev == rmote_event) {
      /* Retrieve sensor data */
      handler(data);
      /* Call event handler */
      env_sensor_event_handler(&resource_env_sensor);
    }

  } /* while (1) */

  PROCESS_END();
}
