/*
 * defs.h
 *
 * Copyright (c) 2012, ROSAND Technologies
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the Institute nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE INSTITUTE AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE INSTITUTE OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * This file is part of the Contiki operating system.
 *
 */

#ifndef _DEFS_H_

#define _DEFS_H_

#include <avr/io.h>

/**
 * \name Macros used to generate read register names from platform-specific definitions of ports.
 * \brief The various CAT macros (DDR, PORT, and PIN) are used to
 * assign port/pin/DDR names to various macro variables.  The
 * variables are assigned based on the specific connections made in
 * the hardware.  For example TCCR(TICKTIMER,A) can be used in place of TCCR0A
 * if TICKTIMER is defined as 0.
 * \{
 */

#define CAT(x, y)      x##y
#define CAT2(x, y, z)  x##y##z
#define DDR(x)         CAT(DDR,  x)
#define PORT(x)        CAT(PORT, x)
#define PIN(x)         CAT(PIN,  x)
#define UCSR(num, let) CAT2(UCSR,num,let)
#define RXEN(x)        CAT(RXEN,x)
#define TXEN(x)        CAT(TXEN,x)
#define TXC(x)         CAT(TXC,x)
#define RXC(x)         CAT(RXC,x)
#define RXCIE(x)       CAT(RXCIE,x)
#define UCSZ(x,y)      CAT2(UCSZ,x,y)
#define UBRR(x,y)      CAT2(UBRR,x,y)
#define UDRE(x)        CAT(UDRE,x)
#define UDRIE(x)       CAT(UDRIE,x)
#define UDR(x)         CAT(UDR,x)
#define TCNT(x)        CAT(TCNT,x)
#define TIMSK(x)       CAT(TIMSK,x)
#define TCCR(x,y)      CAT2(TCCR,x,y)
#define COM(x,y)       CAT2(COM,x,y)
#define OCR(x,y)       CAT2(OCR,x,y)
#define CS(x,y)        CAT2(CS,x,y)
#define WGM(x,y)       CAT2(WGM,x,y)
#define OCIE(x,y)      CAT2(OCIE,x,y)
#define COMPVECT(x)    CAT2(TIMER,x,_COMPA_vect)
#define UDREVECT(x)    CAT2(USART,x,_UDRE_vect)
#define RXVECT(x)      CAT2(USART,x,_RX_vect)

/** \brief Macros defined for HAL_TIMER1.
 *
 *  These macros are used to define the correct setupt of the AVR's Timer1, and
 *  to ensure that the hal_get_system_time function returns the system time in
 *  symbols (16 us ticks).
 */

#if ( F_CPU == 16000000UL )
    #define HAL_TCCR1B_CONFIG ((1 << ICES1) | (1 << CS12))
    #define HAL_US_PER_SYMBOL (1)
    #define HAL_SYMBOL_MASK   (0xFFFFffff)
#elif ( F_CPU == 0x800000UL )
    #define HAL_TCCR1B_CONFIG ((1 << ICES1) | (1 << CS11) | (1 << CS10))
    #define HAL_US_PER_SYMBOL (2)
    #define HAL_SYMBOL_MASK   (0x7FFFffff)
#elif ( F_CPU == 8000000UL )
    #define HAL_TCCR1B_CONFIG ((1 << ICES1) | (1 << CS11) | (1 << CS10))
    #define HAL_US_PER_SYMBOL (2)
    #define HAL_SYMBOL_MASK   (0x7FFFffff)
#elif ( F_CPU == 7954432UL )
    #define HAL_TCCR1B_CONFIG ((1 << ICES1) | (1 << CS11) | (1 << CS10))
    #define HAL_US_PER_SYMBOL (2)
    #define HAL_SYMBOL_MASK   (0x7FFFffff)
#elif ( F_CPU == 4000000UL )
    #define HAL_TCCR1B_CONFIG ((1 << ICES1) | (1 << CS11) | (1 << CS10))
    #define HAL_US_PER_SYMBOL (1)
    #define HAL_SYMBOL_MASK   (0xFFFFffff)
#elif ( F_CPU == 1000000UL )
    #define HAL_TCCR1B_CONFIG ((1 << ICES1) | (1 << CS11))
    #define HAL_US_PER_SYMBOL (2)
    #define HAL_SYMBOL_MASK   (0x7FFFffff)
#else
    #error "Clock speed not supported."
#endif

/* uint32 processing */
#define BREAK_UINT32( var, ByteNum ) \
    (uint8_t)((uint32_t)(((var) >>((ByteNum) * 8)) & 0x00FF))

#define BUILD_UINT32(Byte0, Byte1, Byte2, Byte3) \
  ((uint32_t)((uint32_t)((Byte0) & 0x00FF)       \
      + ((uint32_t)((Byte1) & 0x00FF) << 8)      \
      + ((uint32_t)((Byte2) & 0x00FF) << 16)     \
      + ((uint32_t)((Byte3) & 0x00FF) << 24)))

#define HI_UINT32(a) ((uint16_t) (((uint32_t)(a)) >> 16))
#define LO_UINT32(a) ((uint16_t) ((uint32_t)(a)))

/* uint16 processing */
#define BUILD_UINT16(low, high) \
  ((uint16_t)(((low) & 0x00FF) + (((high) & 0x00FF) << 8)))

#define HI_UINT16(a) (((uint16_t)(a) >> 8) & 0xFF)
#define LO_UINT16(a) ((uint16_t)(a) & 0xFF)


/* uint16 processing */
#define BUILD_UINT8(hiByte, loByte) \
  ((uint8_t)(((loByte) & 0x0F) + (((hiByte) & 0x0F) << 4)))

#define HI_UINT8(a) (((uint8_t)(a) >> 4) & 0x0F)
#define LO_UINT8(a) ((uint8_t)(a) & 0x0F)

#endif /* _DEFS_H_ */
