/*
 * gps-process.c
 *
 * Copyright (c) 2012, ROSAND Technologies
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the Institute nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE INSTITUTE AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE INSTITUTE OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * This file is part of the Contiki operating system.
 *
 */

#include "contiki.h"

#include "sys/clock.h"
#include "dev/rmote-sensors.h"
#include "gps-process.h"
#include "defs.h"

#define RESETPORT               B
#define RESETPIN                5

#define GPS_RESET_DDR           DDR(RESETPORT)
#define GPS_RESET_PORT          PORT(RESETPORT)
#define GPS_RESET_PIN           RESETPIN

static uint16_t lat_l, lat_h, lon_l, lon_h;

/*---------------------------------------------------------------------------*/
PROCESS(gps_sensor_process, "environment server process");
/*---------------------------------------------------------------------------*/
static void
gps_reset(void)
{
  GPS_RESET_DDR  |= (1 << GPS_RESET_PIN);       /* Set GPS Reset as output */
  GPS_RESET_PORT &= ~(1 << GPS_RESET_PIN);      /* Reset the GPS Module */
  clock_delay_usec(100);                        /* Wait 100 usec */
  GPS_RESET_PORT |= (1 << GPS_RESET_PIN);       /* Then run the GPS Module */
}
/*---------------------------------------------------------------------------*/
static void
init(void)
{
  lat_l = lat_h  = 0;
  lon_l = lon_h = 0;

  gps_reset();
  SENSORS_ACTIVATE(gps_sensor);
  process_start(&gps_sensor_process, NULL);
}
/*---------------------------------------------------------------------------*/
static uint16_t
handle(uint8_t *data)
{
  uint8_t *tmp = data;

  *tmp++ = 0;
  *tmp++ = GPS_LAT_SENSOR;
  *tmp++ = GPS_LAT_LENGTH;
  *tmp++ = HI_UINT16(lat_h);
  *tmp++ = LO_UINT16(lat_h);
  *tmp++ = HI_UINT16(lat_l);
  *tmp++ = LO_UINT16(lat_l);
  *tmp++ = GPS_LON_SENSOR;
  *tmp++ = GPS_LON_LENGTH;
  *tmp++ = HI_UINT16(lon_h);
  *tmp++ = LO_UINT16(lon_h);
  *tmp++ = HI_UINT16(lon_l);
  *tmp++ = LO_UINT16(lon_l);
  *data = tmp - data;

  return *data;
}
/*---------------------------------------------------------------------------*/
PROCESS_THREAD(gps_sensor_process, ev, data)
{
  static struct etimer et;

  PROCESS_BEGIN();

  etimer_set(&et, GPS_SENSOR_POLL_INTERVAL * CLOCK_SECOND);

  while(1) {
    uint16_t vlatl, vlath;
    uint16_t vlonl, vlonh;

    vlatl = vlath = vlonl = vlonh = 0;

    PROCESS_WAIT_EVENT_UNTIL(etimer_expired(&et));
    vlatl = gps_sensor.value(GPS_LAT_LOW);
    vlath = gps_sensor.value(GPS_LAT_HIGH);
    vlonl = gps_sensor.value(GPS_LON_LOW);
    vlonh = gps_sensor.value(GPS_LON_HIGH);

    if ((vlatl != lat_l) || (vlath != lat_h) || (vlonl != lon_l) || (vlonh != lon_h)) {
      lon_l = vlonl;
      lon_h = vlonh;
      lat_l = vlatl;
      lat_h = vlath;
      sensors_changed(&gps_sensor);
    }
    etimer_restart(&et);
  }
  PROCESS_END();
}
/*---------------------------------------------------------------------------*/

const struct sensors_handle gps_sensor_handle =
{
    init,
    handle,
};
